#!/bin/env python3

import ctypes
import pydbus

import syslog

import gi
gi.require_version('Gtk', '3.0') 
from gi.repository import GLib, Gtk, Gio

class KdkProcess:
    """
    <node>
        <interface name='com.kylin.kysdk.processsec.process'>
            <method name='SetAntiKilled' >
                <arg type='s' direction='in' name='filepath'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='CancelAntiKilled' >
                <arg type='s' direction='in' name='filepath'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='GetAntiKilled' >
                <arg type='as' direction='out' name='list'/>
            </method>
            <method name='SetExec' >
                <arg type='s' direction='in' name='filepath'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='CancelExec' >
                <arg type='s' direction='in' name='filepath'/>
                <arg type='i' direction='out' name='ret'/>
            </method>

        </interface>
    </node>
    """

    def __init__(self):
        try:
            self.libkyprocesssec = ctypes.CDLL("libkyprocesssec.so")
        except Exception:
            exit(-1)

    def SetAntiKilled(self, filepath):
        rc = self.libkyprocesssec.kdk_process_set_anti_killed(ctypes.c_char_p(bytes(filepath, encoding = "utf8")))
        return rc

    def CancelAntiKilled(self, filepath):
        rc = self.libkyprocesssec.kdk_process_cancel_anti_killed(ctypes.c_char_p(bytes(filepath, encoding = "utf8")))
        return rc

    def GetAntiKilled(self):
        out = []
        count = ctypes.c_int(0)
        self.libkyprocesssec.kdk_process_get_anti_killed.restype = ctypes.POINTER(ctypes.c_char_p)
        plist = self.libkyprocesssec.kdk_process_get_anti_killed(ctypes.byref(count))

        for p in range(0, count.value):
            out.append(str(plist[p], "UTF-8"))
        return out
    
    def SetExec(self, filepath):
        rc = self.libkyprocesssec.kdk_process_set_executable(ctypes.c_char_p(bytes(filepath, encoding = "utf8")))
        return rc

    def CancelExec(self, filepath):
        rc = self.libkyprocesssec.kdk_process_cancel_executable(ctypes.c_char_p(bytes(filepath, encoding = "utf8")))
        return rc

class KdkKmod:
    """
    <node>
        <interface name='com.kylin.kysdk.processsec.kmod'>
            <method name='SetAntiUnloadedPath' >
                <arg type='s' direction='in' name='filepath'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='CancelAntiUnloadedPath' >
                <arg type='s' direction='in' name='filepath'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='GetAntiUnloaded' >
                <arg type='as' direction='out' name='list'/>
            </method>
        </interface>
    </node>
    """

    def __init__(self):
        try:
            self.libkyprocesssec = ctypes.CDLL("libkyprocesssec.so")
        except Exception:
            exit(-1)

    def SetAntiKilled(self, filepath):
        rc = self.libkyprocesssec.kdk_kmod_set_anti_unloaded_by_path(ctypes.c_char_p(bytes(filepath, encoding = "utf8")))
        return rc

    def CancelAntiKilled(self, filepath):
        rc = self.libkyprocesssec.kdk_kmod_cancel_anti_unloaded_by_path(ctypes.c_char_p(bytes(filepath, encoding = "utf8")))
        return rc
    def GetAntiKilled(self):
        out = []
        count = ctypes.c_int(0)
        self.libkyprocesssec.kdk_kmod_get_anti_unloaded.restype = ctypes.POINTER(ctypes.c_char_p)
        plist = self.libkyprocesssec.kdk_kmod_get_anti_unloaded(ctypes.byref(count))

        for p in range(0, count.value):
            out.append(str(plist[p], "UTF-8"))
        return out
    

if __name__ == '__main__':
    loop = GLib.MainLoop()
    bus = pydbus.SystemBus()
    bus.register_object("/com/kylin/kysdk/processsec/process", KdkProcess(), None)
    bus.register_object("/com/kylin/kysdk/processsec/kmod", KdkKmod(), None)
    bus.publish("com.kylin.kysdk.processsec")
    loop.run()